<?php
/**
 * 2021 Floa BANK
 *
 * THE MIT LICENSE
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * @author    FLOA Bank
 * @copyright 2021 FLOA Bank
 * @license   https://opensource.org/licenses/MIT The MIT License
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once 'lib/floapay/autoload.php';

class FloaPay extends PaymentModule
{
    CONST CGV_LINKS = [
        'FR' => 'https://www.floabank.fr/images/pdf/Politique_de_confidentialit_et_cookie.pdf',
        'ES' => 'https://www.floapay.com/images/PDF/20210614_POLITIQUE_CONFIDENTIALITE_FLOA_BANK_vES.pdf',
        'BE' => 'https://www.floapay.com/images/PDF/Politique_de_confidentialit_BELG.pdf',
        'default' => 'https://www.floapay.com/images/PDF/20210330_Politique_de_confidentialit_FLOA_Bank_EN.pdf'
    ];

    public function __construct()
    {
        $this->author      = 'FLOA Bank';
        $this->name        = 'floapay';
        $this->tab         = 'payments_gateways';
        $this->version     = '1.1.0';
        $this->need_instance = false;
        $this->bootstrap   = true;
        $this->displayName = 'FloaPay';
        $this->description = $this->l('Pay in several instalments with your credit card powered by FLOA BANK');
        $this->controllers = [ 'eligibility', 'back', 'return', 'ipn' ];
        $this->ps_versions_compliancy = ['min' => '1.5.6.2', 'max' => _PS_VERSION_];
        parent::__construct();
    }

    public static function getPaymentMethodsAvailable()
    {
        return [
            'CB1XD' => ['ech'  => '1',  'lib' => '1XD', 'floapay', 'paymentOptionRef' => 94],
            'CB3X'  => ['ech'  => '3',  'lib' => '3X',  'floapay', 'paymentOptionRef' => 81],
            'CB4X'  => ['ech'  => '4',  'lib' => '4X',  'floapay', 'paymentOptionRef' => 63],
            'CB10X' => ['ech'  => '10', 'lib' => '10X', 'floapay', 'paymentOptionRef' => 98],
        ];
    }

    public function install()
    {
        $core_install = parent::install();
        if (!$core_install) {
            return false;
        }
        if (version_compare(_PS_VERSION_, '1.7', '>=')) {
            $paymentHooks = ['paymentOptions', 'displayPaymentReturn', 'displayAdminOrder'];
        } else {
            $paymentHooks = ['displayPayment', 'displayPaymentReturn', 'displayAdminOrder'];
        }
        foreach ($paymentHooks as $hook) {
            if (!$this->registerHook($hook)) {
                return false;
            }
        }
        $installSuccessfull = $this->installDb()
            && $this->checkDependencies()
            && $this->registerContextVars()
            && $this->registerHook('displayHeader');
        foreach (array_keys(self::getPaymentMethodsAvailable()) as $payment_key) {
            $installSuccessfull &= $this->registerContextPaymentMethodVars($payment_key);
        }
        Tools::clearCache();
        return (boolean) $installSuccessfull;
    }

    public function uninstall()
    {
        $uninstallSuccessfully = parent::uninstall() && $this->unregisterContextVars();
        foreach (array_keys(self::getPaymentMethodsAvailable()) as $payment_key) {
            $uninstallSuccessfully &= $this->unregisterContextPaymentMethodVars($payment_key);
        }
        return (boolean) $uninstallSuccessfully;
    }

    private function checkDependencies($details = false)
    {
        $result = true;
        $curl = true;
        if (!function_exists('curl_init')) {
            $result = false;
            $curl   = false;
            $this->_errors[] = $this->l('FloaPay requires the CURL PHP extension.', 'floapay');
        }
        $json = true;
        if (!function_exists('json_decode')) {
            $result = false;
            $json   = false;
            $this->_errors[] = $this->l('FloaPay requires the JSON PHP extension.', 'floapay');
        }
        $openssl_exception = $this->l('FloaPay requires OpenSSL >= 1.0.1', 'floapay');
        $ssl = true;
        if (!defined('OPENSSL_VERSION_TEXT')) {
            $result = false;
            $ssl    = false;
            $this->_errors[] = $openssl_exception;
        }
        preg_match('/^(?:Libre|Open)SSL ([\d.]+)/', OPENSSL_VERSION_TEXT, $matches);
        if (empty($matches[1])) {
            $result = false;
            $ssl    = false;
            $this->_errors[] = $openssl_exception;
        }
        if (!version_compare($matches[1], '1.0.1', '>=')) {
            $result = false;
            $ssl    = false;
            $this->_errors[] = $openssl_exception;
        }
        if ($details) {
            return array(
                'curl' => array('state' => $curl, 'message' => $this->l('CURL PHP extension', 'floapay')),
                'json' => array('state' => $json, 'message' => $this->l('JSON PHP extension', 'floapay')),
                'ssl'  => array('state' => $ssl,  'message' => $this->l('OpenSSL extension', 'floapay')),
            );
        } else {
            return $result;
        }
    }

    public function installDb()
    {
        $return = true;
        $sql = require_once(dirname(__FILE__) . '/sql/install.php');
        if ($sql) {
            foreach ($sql as $s) {
                $return &= Db::getInstance()->execute($s);
            }
        }
        return (boolean) $return;
    }

    public function registerContextVars()
    {
        self::updateConfKey('FLOA_ENV', 'INTE');
        self::updateConfKey('FLOA_MERCHANT_LOGIN', '');
        self::updateConfKey('FLOA_MERCHANT_PASSWORD', '');
        self::updateConfKey('FLOA_MERCHANT_COUNTRY', 'FR');
        self::updateConfKey('FLOA_MERCHANT_ID', '');
        self::updateConfKey('FLOA_TK_TOKEN', '');
        self::updateConfKey('FLOA_TK_TOKEN_UPDATE', time());
        self::updateConfKey('FLOA_TK_TOKEN_ENV', '');
        return true;
    }

    public function unregisterContextVars()
    {
        Configuration::deleteByName('FLOA_ENV');
        Configuration::deleteByName('FLOA_MERCHANT_LOGIN');
        Configuration::deleteByName('FLOA_MERCHANT_PASSWORD');
        Configuration::deleteByName('FLOA_MERCHANT_COUNTRY', '');
        Configuration::deleteByName('FLOA_MERCHANT_ID');
        Configuration::deleteByName('FLOA_TK_TOKEN');
        Configuration::deleteByName('FLOA_TK_TOKEN_UPDATE');
        Configuration::deleteByName('FLOA_TK_TOKEN_ENV');
        return true;
    }

    public function registerContextPaymentMethodVars($type)
    {
        self::updateConfKey('FLOA_' . $type . '_ENABLED', 'false');
        self::updateConfKey('FLOA_' . $type . '_MERCHANT_SITE_ID', '');
        if ($type == "CB1XD") {
            self::updateConfKey('FLOA_' . $type . '_REPORT_MODE', '1');
            self::updateConfKey('FLOA_' . $type . '_REPORT_DELAY', '0');
        }
        return true;
    }

    public function unregisterContextPaymentMethodVars($type)
    {
        Configuration::deleteByName('FLOA_' . $type . '_ENABLED');
        Configuration::deleteByName('FLOA_' . $type . '_MERCHANT_SITE_ID');
        if ($type == "CB1XD") {
            Configuration::deleteByName('FLOA_' . $type . '_REPORT_MODE');
            Configuration::deleteByName('FLOA_' . $type . '_REPORT_DELAY');
        }
        return true;
    }

    public function getContent()
    {
        $this->html = $this->postProcess();
        $this->context->controller->addCSS($this->_path.'views/css/admin/floapay.css', 'all');
        $payment_methods_checker = array();
        foreach (array_keys(self::getPaymentMethodsAvailable()) as $payment_key) {
            if (self::getConfKey('FLOA_'.$payment_key.'_ENABLED') == 'true') {
                $test = new floaPayApi($payment_key);
                $payment_methods_checker[$payment_key] = $test->contextCheck;
            } else {
                $payment_methods_checker[$payment_key] = array(
                    'state'   => false,
                    'active'  => false,
                    'message' => array($this->l('This payment method is disabled', 'floapay')),
                );
            }
        }
        $this->context->smarty->assign(array(
            'module_icon_path'  => $this->getPathUri().'views/img/logo_solution.png',
            'states_list'       => $this->checkDependencies(true),
            'env'               => self::getConfKey('FLOA_ENV'),
            'payment_methods'   => $payment_methods_checker,
            'fp_version'        => version_compare(_PS_VERSION_, '1.6', '>=') ? 'new' : 'old',
            'country'           => self::getConfKey('FLOA_MERCHANT_COUNTRY'),
            'cultures'          => $this->allowedCultureByCountry(self::getConfKey('FLOA_MERCHANT_COUNTRY'))
        ));
        $this->html .= $this->context->smarty->fetch($this->local_path.'views/templates/admin/configure.tpl');
        $this->html .= $this->renderForm();
        return $this->html;
    }

    public function renderForm()
    {
        $forms_list = array();
        $forms_list['global']   = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Merchant configuration', 'floapay'),
                    'image'  => $this->_path . '/views/img/logos/floa_tiny.png'
                ),
                'input'  => array()
            )
        );
        $countryList = [];
        foreach ($this->allowedCountry() as $country => $name) {
            $countryList[] = [
                'id_option' => $country,
                'name' => $name
            ];
        }
        $forms_list['global']['form']['input'] = array(
            array(
                'type'          => 'text',
                'label'         => $this->l('Login', 'floapay'),
                'name'          => 'FLOA_MERCHANT_LOGIN',
                'required'      => true,
                'empty_message' => $this->l('This field is required', 'floapay'),
            ),
            array(
                'type'          => 'text',
                'label'         => $this->l('Password', 'floapay'),
                'name'          => 'FLOA_MERCHANT_PASSWORD',
                'required'      => true,
                'empty_message' => $this->l('This field is required', 'floapay'),
            ),
            array(
                'type'          => 'text',
                'label'         => $this->l('Merchant ID', 'floapay'),
                'name'          => 'FLOA_MERCHANT_ID',
                'required'      => true,
                'empty_message' => $this->l('This field is required', 'floapay'),
            ),
            array(
                'type'          => 'select',
                'class'         => 'floa-select',
                'label'         => $this->l('Country', 'floapay'),
                'name'          => 'FLOA_MERCHANT_COUNTRY',
                'required'      => true,
                'empty_message' => $this->l('This field is required', 'floapay'),
                'options'       => array(
                    'query' => $countryList,
                    'id'   => 'id_option',
                    'name' => 'name',
                )
            ),
            array(
                'type'          => 'radio',
                'class'         => 'floa-radio',
                'label'         => $this->l('Environment', 'floapay'),
                'name'          => 'FLOA_ENV',
                'required'      => true,
                'empty_message' => $this->l('This field is required', 'floapay'),
                'values'        => array(
                    array(
                        'id'    => 'integration',
                        'value' => 'INTE',
                        'label' => $this->l('Integration', 'floapay')
                    ),
                    array(
                        'id'    => 'production',
                        'value' => 'PROD',
                        'label' => $this->l('Production', 'floapay')
                    )
                )
            )
        );
        foreach (self::getPaymentMethodsAvailable() as $payment_key => $payment_method) {
            $iconPath = $this->_path . '/views/img/logos/floa_'.Tools::strtoupper($payment_key).'_tiny.png';
            $forms_list[$payment_key] = array(
                'form' => array(
                    'legend' => array(
                        'title' => $this->l('Configuration', 'floapay').' '.$payment_method['lib'],
                        'image' => $iconPath,
                    ),
                    'input'  => array(),
                )
            );
            $forms_list[$payment_key]['form']['input'] = array(
                array(
                    'type'          => 'radio',
                    'class'         => 'floa-radio',
                    'label'         => $this->l('Enable', 'floapay'),
                    'name'          => 'FLOA_'.$payment_key.'_ENABLED',
                    'required'      => true,
                    'empty_message' => $this->l('This field is required', 'floapay'),
                    'values'        => array(
                        array(
                            'id'    => 'no',
                            'value' => 'false',
                            'label' => $this->l('No', 'floapay')
                        ),
                        array(
                            'id'    => 'yes',
                            'value' => 'true',
                            'label' => $this->l('Yes', 'floapay')
                        )
                    ),
                ),
                array(
                    'type'          => 'text',
                    'label'         => $this->l('Merchant Site ID', 'floapay'),
                    'name'          => 'FLOA_'.$payment_key.'_MERCHANT_SITE_ID',
                    'required'      => true,
                    'empty_message' => $this->l('This field is required', 'floapay'),
                )
            );
            if ($payment_key == "CB1XD") {
                $forms_list[$payment_key]['form']['input'][] = array(
                    'type'          => 'radio',
                    'class'         => 'floa-radio',
                    'label'         => $this->l('Report type', 'floapay'),
                    'name'          => 'FLOA_'.$payment_key.'_REPORT_MODE',
                    'required'      => true,
                    'empty_message' => $this->l('This field is required', 'floapay'),
                    'values'        => array(
                        array(
                            'id'    => 'customer',
                            'value' => '1',
                            'label' => $this->l('Customer', 'floapay')
                        ),
                        array(
                            'id'    => 'fixed',
                            'value' => '2',
                            'label' => $this->l('Fixed', 'floapay')
                        )
                    ),
                );
                $forms_list[$payment_key]['form']['input'][] = array(
                    'type'          => 'text',
                    'label'         => $this->l('Report delay', 'floapay'),
                    'name'          => 'FLOA_'.$payment_key.'_REPORT_DELAY',
                    'suffix'        => $this->l('days', 'floapay')
                );
            }
        }
        $carrierMappingModel = new floaCarrierMapping();
        $allPsCarrier        = $carrierMappingModel->getAll();
        $allCb4xCarrier      = $carrierMappingModel->getList();
        $forms_list['carriers'] = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Association of delivery methods', 'floapay'),
                    'icon'  => 'icon-truck'
                ),
                'input'  => array()
            ),
        );
        if ($allPsCarrier) {
            $query = array();
            foreach ($allCb4xCarrier as $keyCb4xCarrier => $labelCb4xCarrier) {
                $query[] = array(
                    'id_option' => $keyCb4xCarrier,
                    'name'      => $labelCb4xCarrier,
                );
            }
            foreach ($allPsCarrier as $iCarrierId => $psCarrier) {
                $forms_list['carriers']['form']['input'][$iCarrierId] = array(
                    'type'          => 'select',
                    'label'         => $psCarrier['name'],
                    'name'          => "FLOA_CARRIER_" . $psCarrier['ps_id'],
                    'required'      => true,
                    'empty_message' => $this->l('This field is required', 'floapay'),
                    'options'       => array(
                        'query' => $query,
                        'id'   => 'id_option',
                        'name' => 'name',
                    ),
                );
            }
        }
        $forms_list['validate']   = array(
            'form' => array(
                'submit' => array(
                    'title' => $this->l('Save', 'floapay'),
                    'class' => 'button btn btn-default pull-right submit',
                )
            )
        );
        $helper                           = new HelperForm();
        $helper->show_toolbar             = false;
        $helper->default_form_language    = (int)self::getConfKey('PS_LANG_DEFAULT');
        $helper->allow_employee_form_lang = self::getConfKey(
            'PS_BO_ALLOW_EMPLOYEE_FORM_LANG'
        ) ? self::getConfKey('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $helper->id                       = $this->identifier;
        $helper->identifier               = $this->identifier;
        $helper->submit_action            = 'SubmitPaymentConfiguration';
        $helper->currentIndex             = $this->context->link->getAdminLink(
            'AdminModules',
            false
        ).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token                    = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars                 = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages'    => $this->context->controller->getLanguages(),
            'id_language'  => $this->context->language->id
        );
        return $helper->generateForm($forms_list);
    }

    public function getConfigFieldsValues()
    {
        $response = false;
        $response['FLOA_MERCHANT_LOGIN']
            = Tools::getValue('FLOA_MERCHANT_LOGIN', self::getConfKey('FLOA_MERCHANT_LOGIN'));
        $response['FLOA_MERCHANT_PASSWORD']
            = Tools::getValue('FLOA_MERCHANT_PASSWORD', self::getConfKey('FLOA_MERCHANT_PASSWORD'));
        $response['FLOA_MERCHANT_ID']
            = Tools::getValue('FLOA_MERCHANT_ID', self::getConfKey('FLOA_MERCHANT_ID'));
        $response['FLOA_MERCHANT_COUNTRY']
            = Tools::getValue('FLOA_MERCHANT_COUNTRY', self::getConfKey('FLOA_MERCHANT_COUNTRY'));
        $response['FLOA_ENV'] = Tools::getValue('FLOA_ENV', self::getConfKey('FLOA_ENV'));
        foreach (array_keys(self::getPaymentMethodsAvailable()) as $payment_key) {
            $response['FLOA_'.$payment_key.'_MERCHANT_SITE_ID'] = Tools::getValue(
                'FLOA_'.$payment_key.'_MERCHANT_SITE_ID',
                self::getConfKey('FLOA_'.$payment_key.'_MERCHANT_SITE_ID')
            );
            $response['FLOA_'.$payment_key.'_ENABLED'] = Tools::getValue(
                'FLOA_'.$payment_key.'_ENABLED',
                self::getConfKey('FLOA_'.$payment_key.'_ENABLED')
            );
            if ($payment_key == "CB1XD") {
                $response['FLOA_'.$payment_key.'_REPORT_MODE'] = Tools::getValue(
                    'FLOA_'.$payment_key.'_REPORT_MODE',
                    self::getConfKey('FLOA_'.$payment_key.'_REPORT_MODE')
                );
                $response['FLOA_'.$payment_key.'_REPORT_DELAY'] = Tools::getValue(
                    'FLOA_'.$payment_key.'_REPORT_DELAY',
                    self::getConfKey('FLOA_'.$payment_key.'_REPORT_DELAY')
                );
            }
        }
        $carrierMappingModel = new floaCarrierMapping();
        $allPsCarrier        = $carrierMappingModel->getAll();
        if ($allPsCarrier) {
            foreach ($allPsCarrier as $psCarrier) {
                $response["FLOA_CARRIER_" . $psCarrier['ps_id']] 
                    = $carrierMappingModel->getFloaIdByPsId($psCarrier['ps_id']);
            }
        }
        return $response;
    }
    
    public function postProcess()
    {
        $carrierMappingModel = new floaCarrierMapping();
        $allPsCarrier        = $carrierMappingModel->getAll();
        if (Tools::isSubmit('SubmitPaymentConfiguration')) {
            foreach ($allPsCarrier as $psCarrier) {
                $existCarrier = $carrierMappingModel->getIdByPsId($psCarrier['ps_id']);
                $carrierMappingItem = new floaCarrierMapping();
                if ($existCarrier > 0) {
                    $carrierMappingItem->id = $existCarrier;
                }
                $carrierMappingItem->ps_id   = $psCarrier['ps_id'];
                $carrierMappingItem->floa_id = Tools::getValue("FLOA_CARRIER_" . $psCarrier['ps_id']);
                $carrierMappingItem->save();
            }
            $bAsError = false;
            if (Tools::strlen(Tools::getValue('FLOA_MERCHANT_LOGIN')) < 3
                || Tools::strlen(Tools::getValue('FLOA_MERCHANT_LOGIN')) > 64) {
                $this->context->controller->errors[] = $this->l('Login field is invalid.', 'floapay');
                $bAsError = true;
            }
            if (Tools::strlen(Tools::getValue('FLOA_MERCHANT_PASSWORD')) < 3
                || Tools::strlen(Tools::getValue('FLOA_MERCHANT_PASSWORD')) > 64) {
                $this->context->controller->errors[] = $this->l('Password field is invalid.', 'floapay');
                $bAsError = true;
            }
            if (!is_numeric(Tools::getValue('FLOA_MERCHANT_ID'))) {
                $this->context->controller->errors[] = $this->l('Merchant ID field is invalid.', 'floapay');
                $bAsError = true;
            }
            if (!array_key_exists(Tools::getValue('FLOA_MERCHANT_COUNTRY'), $this->allowedCountry())) {
                $this->context->controller->errors[] = $this->l('Country field is invalid.', 'floapay');
                $bAsError = true;
            }
            if (!in_array(Tools::getValue('FLOA_ENV'), array('INTE', 'PROD'))) {
                $this->context->controller->errors[] = $this->l('Environement field is invalid.', 'floapay');
                $bAsError = true;
            }
            foreach (self::getPaymentMethodsAvailable() as $payment_key => $payment_method) {
                if (!in_array(Tools::getValue('FLOA_'.$payment_key.'_ENABLED'), array('true', 'false'))) {
                    $this->context->controller->errors[]
                        = $this->l('Enable field is invalid for', 'floapay').' '.$payment_method['lib'];
                    $bAsError = true;
                }
                if (!is_numeric(Tools::getValue('FLOA_'.$payment_key.'_MERCHANT_SITE_ID'))
                    && Tools::getValue('FLOA_'.$payment_key.'_ENABLED') == 'true') {
                    $this->context->controller->errors[]
                        = $this->l('Merchant Site ID field is invalid for', 'floapay').' '.$payment_method['lib'];
                    $bAsError = true;
                }
                if ($payment_key == "CB1XD") {
                    if (!in_array(Tools::getValue('FLOA_'.$payment_key.'_REPORT_MODE'), array('1', '2'))
                        && Tools::getValue('FLOA_'.$payment_key.'_ENABLED') == 'true') {
                        $this->context->controller->errors[]
                            = $this->l('Report mode field is invalid for', 'floapay').' '.$payment_method['lib'];
                        $bAsError = true;
                    }
                    if (!is_numeric(Tools::getValue('FLOA_'.$payment_key.'_REPORT_DELAY'))
                        && Tools::getValue('FLOA_'.$payment_key.'_ENABLED') == 'true') {
                        $this->context->controller->errors[]
                            = $this->l('Report delay field is invalid for', 'floapay').' '.$payment_method['lib'];
                        $bAsError = true;
                    }
                }
            }
            if (!$bAsError) {
                self::updateConfKey('FLOA_MERCHANT_LOGIN', Tools::getValue('FLOA_MERCHANT_LOGIN'));
                self::updateConfKey('FLOA_MERCHANT_PASSWORD', Tools::getValue('FLOA_MERCHANT_PASSWORD'));
                self::updateConfKey('FLOA_MERCHANT_ID', Tools::getValue('FLOA_MERCHANT_ID'));
                self::updateConfKey('FLOA_MERCHANT_COUNTRY', Tools::getValue('FLOA_MERCHANT_COUNTRY'));
                self::updateConfKey('FLOA_ENV', Tools::getValue('FLOA_ENV'));
                self::updateConfKey('FLOA_TK_TOKEN', '');
                self::updateConfKey('FLOA_TK_TOKEN_UPDATE', time());
                self::updateConfKey('FLOA_TK_TOKEN_ENV', '');
                foreach (self::getPaymentMethodsAvailable() as $payment_key => $payment_method) {
                    self::updateConfKey(
                        'FLOA_'.$payment_key.'_ENABLED',
                        Tools::getValue('FLOA_'.$payment_key.'_ENABLED')
                    );
                    self::updateConfKey(
                        'FLOA_'.$payment_key.'_MERCHANT_SITE_ID',
                        Tools::getValue('FLOA_'.$payment_key.'_MERCHANT_SITE_ID')
                    );
                    if ($payment_key == "CB1XD") {
                        self::updateConfKey(
                            'FLOA_'.$payment_key.'_REPORT_MODE',
                            Tools::getValue('FLOA_'.$payment_key.'_REPORT_MODE')
                        );
                        self::updateConfKey(
                            'FLOA_'.$payment_key.'_REPORT_DELAY',
                            Tools::getValue('FLOA_'.$payment_key.'_REPORT_DELAY')
                        );
                    }
                }
                return $this->displayConfirmation($this->l('The configuration has been correctly updated.', 'floapay'));
            }
        }
    }

    public function hookDisplayHeader()
    {
        $this->context->controller->addCSS($this->_path.'views/css/front/floapay.css');
        $this->context->controller->addJS($this->_path.'views/js/front/commonPaymentFloa.js');
        if (version_compare(_PS_VERSION_, '1.7', '>=')) {
            $this->context->controller->addJS($this->_path.'views/js/front/paymentOptionsFloa.js');
        } else {
            $this->context->controller->addJS($this->_path.'views/js/front/displayPaymentFloa.js');
        }
    }

    public function hookDisplayAdminOrder($params)
    {
        $order = new Order($params['id_order']);
        if (Tools::strtolower($order->module) != $this->name) {
            return false;
        }
        $orderDetails   = floaOrder::getByOrderId($order->id);
        $errorMessage   = '';
        $successMessage = '';
        if ($orderDetails) {
            $floaPayAPI = new floaPayApi($orderDetails['op'], $orderDetails['merchant_id'], $orderDetails['merchant_site_id']);
            if ($floaPayAPI->contextCheck['state'] == true) {
                $orderWithoutFeesBeforeRefund = $orderDetails['total_amount']-$orderDetails['fees_amount'];
                if (Tools::isSubmit('floaOrderAction') && floaTools::isTokenValid(Tools::getValue('floaToken'))) {
                    if (Tools::isSubmit('floaOrderAction') && Tools::getValue('floaOrderAction') == "refund") {
                        $amountToRefund = floaTools::convertToInt(Tools::getValue('floaRefundAmount'));
                        if ($amountToRefund > 0 && $amountToRefund <= $orderWithoutFeesBeforeRefund) {
                            $amountAfterRefund = $orderWithoutFeesBeforeRefund-$amountToRefund;
                            $updateOrder = $floaPayAPI->updateOrder(
                                $orderDetails['order_reference'],
                                $orderDetails['scoring_token'],
                                $orderWithoutFeesBeforeRefund,
                                $amountAfterRefund,
                                $orderDetails['merchant_id'],
                                $orderDetails['merchant_site_id']
                            );
                            if ($updateOrder['state'] == true) {
                                floaOrder::updatePartialCancels(
                                    $orderDetails['id'],
                                    $orderDetails['partial_cancels'],
                                    $amountToRefund
                                );
                                $floaOrderUpdate                = new floaOrder($orderDetails['id']);
                                $floaOrderUpdate->id            = $orderDetails['id'];
                                $floaOrderUpdate->total_amount  = $amountAfterRefund+$orderDetails['fees_amount'];
                                $floaOrderUpdate->update();
                                $successMessage = sprintf(
                                    $this->l('Your refund of %s has been registered.', 'floapay'),
                                    Tools::displayPrice(floaTools::convertToFloat($amountToRefund))
                                );
                            } else {
                                $errorMessage = $updateOrder['message'];
                            }
                        } else {
                            $errorMessage = $this->l('The amount to be refunded is incorrect', 'floapay');
                        }
                    } elseif (Tools::isSubmit('floaOrderAction') && Tools::getValue('floaOrderAction') == "pay") {
                        $paymentInfo = $floaPayAPI->getPaymentSchedules(
                            $orderDetails['order_reference'],
                            $orderDetails['merchant_id'],
                            $orderDetails['merchant_site_id']
                        );
                        if (isset($paymentInfo['schedules'])
                            && $paymentInfo['schedules'][0]->state == "init"
                            && !in_array($orderDetails['op'], ['CB1XD', 'CB10X'])) {
                            $capture = $floaPayAPI->payOrderRank(
                                $orderDetails['order_reference'],
                                $orderDetails['merchant_id'],
                                $orderDetails['merchant_site_id']
                            );
                            if ($capture['state'] == true) {
                                $successMessage = $this->l('Order successfully captured.', 'floapay');
                            } else {
                                $errorMessage = $this->l('Unable to capture the order.', 'floapay');
                            }
                        } else {
                            $errorMessage = $this->l('Unable to capture the order.', 'floapay');
                        }
                    }
                } elseif (Tools::isSubmit('floaOrderAction')) {
                    $errorMessage = $this->l('Your token is invalid', 'floapay');
                }
                $orderDetails = floaOrder::getByOrderId($order->id);
                $orderWithoutFees = $orderDetails['total_amount']-$orderDetails['fees_amount'];
                $paymentInfo = $floaPayAPI->getPaymentSchedules(
                    $orderDetails['order_reference'],
                    $orderDetails['merchant_id'],
                    $orderDetails['merchant_site_id']
                );
                $floaToken = floaTools::getToken();
                $orderRefunds
                    = isset($orderDetails['partial_cancels']) ? unserialize($orderDetails['partial_cancels']) : false;
                $this->context->smarty->assign([
                    'apiState'          => $floaPayAPI->contextCheck['state'],
                    'orderDetails'      => $orderDetails,
                    'schedules'         => $paymentInfo['schedules'],
                    'orderRefunds'      => $orderRefunds,
                    'orderWithoutFees'  => $orderWithoutFees,
                    'errorMessage'      => $errorMessage,
                    'successMessage'    => $successMessage,
                    'floaToken'         => $floaToken,
                    'module_icon_path'  => $this->getPathUri().'views/img/logos/',
                ]);
                return (version_compare(_PS_VERSION_, '1.6', '>=')
                    ? $this->context->smarty->fetch(dirname(__FILE__).'/views/templates/admin/hookAdminOrder.tpl')
                    : $this->context->smarty->fetch(dirname(__FILE__).'/views/templates/admin/hookAdminOrder15.tpl'));
            }
        }
    }

    public function hookDisplayPaymentReturn($params)
    {
        if (!$this->active) {
            return false;
        }
        $order          = array_key_exists('objOrder', $params) ? $params['objOrder'] : $params['order'];
        $orderDetails   = floaOrder::getByOrderId($order->id);
        $floaPayApi     = new floaPayApi($orderDetails['op']);
        $paymentInfo    = $floaPayApi->getPaymentSchedules(
            $orderDetails['order_reference'],
            $orderDetails['merchant_id'],
            $orderDetails['merchant_site_id']
        );
        $this->smarty->assign([
            'schedules' => $paymentInfo['schedules'],
            'path_logo' => $this->getPathUri().'views/img/logos/floa_tiny.png'
        ]);
        return $this->display(__FILE__, 'return.tpl');
    }

    public function hookPaymentOptions()
    {
        return $this->runHookPayment('paymentOptions');
    }

    public function hookDisplayPayment()
    {
        return $this->runHookPayment('displayPayment');
    }

    public function runHookPayment($type)
    {
        $this->modulePathFolder = $this->_path;
        $arraysOption   = false;
        $this->l('Refusal reason %s%s', 'floapay');
        $this->l('deferred up to 30 days', 'floapay');
        $this->l('deferred for %s days', 'floapay');
        $this->l('with floa', 'floapay');
        $this->l('Your session is no longer valid, please refresh the page', 'floapay');
        $this->l('The payment method is no longer available', 'floapay');
        $this->l('Civility is invalid.', 'floapay');
        $this->l('Birth date is invalid.', 'floapay');
        $this->l('Postal code is invalid.', 'floapay');
        $this->l('Phone number is invalid.', 'floapay');
        $this->l('You must be over 18 years old.', 'floapay');
        $this->l('Maiden name is invalid.', 'floapay');
        $this->l('Second lastname is invalid.', 'floapay');
        $this->l('Payment date is invalid.', 'floapay');
        $this->l('NIF is invalid.', 'floapay');
        if ($type == "paymentOptions") {
            if (!$this->active) {
                return false;
            }
            foreach (self::getPaymentMethodsAvailable() as $payment_key => $payment_method) {
                $reportDelayInDays = false;
                if ($payment_key == "CB1XD" && FloaPay::getConfKey('FLOA_CB1XD_REPORT_MODE') == 1) {
                    $reportDelayInDays = 30;
                } elseif ($payment_key == "CB1XD" && FloaPay::getConfKey('FLOA_CB1XD_REPORT_MODE') == 2) {
                    $reportDelayInDays = FloaPay::getConfKey('FLOA_CB1XD_REPORT_DELAY');
                }
                sprintf(
                    $this->l('Pay in %s instalment%s', 'floapay'),
                    $payment_method['ech'],
                    ($payment_method['ech'] > 1 ? 's' : '')
                );
                $hookPaymentOptionsFloa = new hookPaymentOptionsFloa();
                $paymentOption = $hookPaymentOptionsFloa->run(
                    $payment_key,
                    $payment_method['ech'],
                    $this->context->cart,
                    $reportDelayInDays
                );
                if ($paymentOption) {
                    $arraysOption[] = $paymentOption;
                }
            }
            return $arraysOption;
        } elseif ($type == "displayPayment") {
            foreach (self::getPaymentMethodsAvailable() as $payment_key => $payment_method) {
                $reportDelayInDays = false;
                if ($payment_key == "CB1XD" && FloaPay::getConfKey('FLOA_CB1XD_REPORT_MODE') == 1) {
                    $reportDelayInDays = 30;
                } elseif ($payment_key == "CB1XD" && FloaPay::getConfKey('FLOA_CB1XD_REPORT_MODE') == 2) {
                    $reportDelayInDays = FloaPay::getConfKey('FLOA_CB1XD_REPORT_DELAY');
                }
                $hookDisplayPaymentFloa = new hookDisplayPaymentFloa();
                $paymentOption = $hookDisplayPaymentFloa->run(
                    $payment_key,
                    $payment_method['ech'],
                    $this->context->cart,
                    $reportDelayInDays
                );
                if ($paymentOption) {
                    $arraysOption[] = $paymentOption;
                }
            }
            $this->smarty->assign(array(
                'arraysOption' => $arraysOption
            ));
            return $this->display(__FILE__, 'displayPayment.tpl');
        }
    }

    public function getLinkByCountry($iso)
    {
        $cvg = self::CGV_LINKS;
        return isset($cvg[Tools::strtoupper($iso)]) ? $cvg[Tools::strtoupper($iso)] : $cvg['default'];
    }

    public static function getConfKey($configKey, $default = null)
    {
        $idShop = Shop::getContextShopID(true);
        $idShopGroup = Shop::getContextShopGroupID(true);
        $value = Configuration::get($configKey, null, $idShopGroup, $idShop, $default);
        if (!$value && !Configuration::hasKey($configKey, null, $idShopGroup, $idShop)) {
            $value = $default;
        }
        return $value;
    }

    public static function updateConfKey($configKey, $value)
    {
        $idShop = Shop::getContextShopID(true);
        $idShopGroup = Shop::getContextShopGroupID(true);
        Configuration::updateValue($configKey, $value, false, $idShopGroup, $idShop);
    }

    public function smartyAssign($array)
    {
        $this->smarty->assign($array);
    }
    
    /**
     * allowedCultureByCountry
     *
     * @param  mixed $selected_country
     * @return void
     */
    public function allowedCultureByCountry($selected_country)
    {
        $selected_country = FloaTools::getCountryCode($selected_country);
        $culturesByCountry = [
            'FR' => ['fr-FR' => false],
            'IT' => ['it-IT' => false],
            'BE' => ['fr-BE' => false, 'nl-BE' => false],
            'PT' => ['pt-PT' => false],
            'ES' => ['es-ES' => false],
        ];
        $available_languages    = Language::getLanguages(true, Context::getContext()->shop->id);
        foreach ($available_languages as $available_language){
            $language_code      = explode('-', $available_language['language_code']);
            $culture = Tools::strtolower($language_code[0]).'-'.Tools::strtoupper($selected_country);
            if (array_key_exists($culture, $culturesByCountry[$selected_country])) {
                $culturesByCountry[$selected_country][$culture] = true;
            } else {
                $culturesByCountry[$selected_country][$culture] = false;
            }
        }
        return $culturesByCountry[$selected_country];
    }
    
    /**
     * allowedCountry
     *
     * @return array
     */
    private function allowedCountry()
    {
        return [
            'FR' => $this->l('France', 'floapay'),
            'IT' => $this->l('Italy', 'floapay'),
            'BE' => $this->l('Belgium', 'floapay'),
            'PT' => $this->l('Portugal', 'floapay'),
            'ES' => $this->l('Spain', 'floapay')
        ];
    }

}
