<?php
/**
 * 2021 Floa BANK
 *
 * THE MIT LICENSE
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * @author    FLOA Bank
 * @copyright 2021 FLOA Bank
 * @license   https://opensource.org/licenses/MIT The MIT License
 */

class floaPayApi
{

    const PAYMENT_GATEWAY_INTE = 'https://paymentgateway.integration-cb4x.fr/';
    const PAYMENT_GATEWAY_PROD = 'https://paymentgateway.cb4x.fr/';

    const ELIGIBILITY_INTE = 'https://eligibility.integration-cb4x.fr/';
    const ELIGIBILITY_PROD = 'https://eligibility.cb4x.fr/';

    const SERVICES_INTE = 'https://paymentservices.integration-cb4x.fr/MerchantGatewayFrontService.svc/';
    const SERVICES_PROD = 'https://paymentservices.cb4x.fr/MerchantGatewayFrontService.svc/';
    
    /**
     * @var string
     */
    private $method;

    /**
     * @var string
     */
    private $merchantLogin;

    /**
     * @var string
     */
    private $merchantPassword;

    /**
     * @var string
     */
    private $env;

    /**
     * @var Date
     */
    private $tokenUpdateDate;

    /**
     * @var string
     */
    private $tokenEnv;

    /**
     * @var string
     */
    private $authorization;

    /**
     * @var string
     */
    private $contextPaymentUri;

    /**
     * @var string
     */
    private $contextEligibilityUri;

    /**
     * @var integer
     */
    private $defaultPaymentOptionRef;

    /**
     * @var object
     */
    private $paymentOption;

    /**
     * @var integer
     */
    private $timeoutMs;
        
    /**
     * @var integer
     */
    public $minAmount;

    /**
     * @var integer
     */
    public $maxAmount;

    /**
     * @var integer
     */
    public $merchantId;

    /**
     * @var string
     */
    public $token;

    /**
     * @var integer
     */
    public $active;

    /**
     * @var integer
     */
    public $merchantSiteId;

    /**
     * @var array
     */
    public $contextCheck;
    
    /**
     * __construct
     *
     * @param  mixed $method
     * @return void
     */
    public function __construct($method, $merchantId = NULL, $merchantSiteId = NULL)
    {
        $this->module                   = Module::getInstanceByName("floapay");
        $this->method                   = strtoupper($method);
        $this->merchantLogin            = FloaPay::getConfKey('FLOA_MERCHANT_LOGIN');
        $this->merchantPassword         = FloaPay::getConfKey('FLOA_MERCHANT_PASSWORD');
        $this->merchantId               = $merchantId != NULL ? $merchantId : FloaPay::getConfKey('FLOA_MERCHANT_ID');
        $this->env                      = FloaPay::getConfKey('FLOA_ENV');
        $this->token                    = FloaPay::getConfKey('FLOA_TK_TOKEN');
        $this->tokenUpdateDate          = FloaPay::getConfKey('FLOA_TK_TOKEN_UPDATE');
        $this->tokenEnv                 = FloaPay::getConfKey('FLOA_TK_TOKEN_ENV');
        $this->active                   = FloaPay::getConfKey('FLOA_'.$this->method.'_ENABLED');
        $this->merchantSiteId           = $merchantSiteId != NULL ? $merchantSiteId : FloaPay::getConfKey('FLOA_'.$this->method.'_MERCHANT_SITE_ID');
        $this->authorization            = base64_encode($this->merchantLogin.':'.$this->merchantPassword);
        $this->contextPaymentUri        = $this->env == "INTE" ? self::PAYMENT_GATEWAY_INTE : self::PAYMENT_GATEWAY_PROD;
        $this->contextEligibilityUri    = $this->env == "INTE" ? self::ELIGIBILITY_INTE : self::ELIGIBILITY_PROD;
        $this->contextServicesUri       = $this->env == "INTE" ? self::SERVICES_INTE : self::SERVICES_PROD;
        $this->defaultPaymentOptionRef  = floaPay::getPaymentMethodsAvailable()[$this->method]['paymentOptionRef'];
        $this->timeoutMs                = 30000;
        $this->contextCheck = $this->checkConfig();
    }
    
    /**
     * checkConfig
     *
     * Check the context configuration
     * 
     * @return array
     */
    public function checkConfig()
    {

        $configValid  = true;
        $errors       = false;
        if (!$this->merchantSiteId)
        {
            $configValid = false;
            $errors[] = 'The Merchant Site ID is invalid.';
        }
        if (!$this->merchantId)
        {
            $configValid = false;
            $errors[] = 'The Merchant ID is invalid.';
        }
        if ($this->active == "false")
        {
            return ['state' => false, 'message' => 'The Merchant Site ID is not active.'];
        } else {
            if (!$this->merchantLogin || !$this->merchantPassword)
            {
                $configValid = false;
                $errors[] = 'The credentials are invalids.';
            }
            $tokenContext = $this->getSecurityToken();
            if ($tokenContext['state'] == false && $configValid)
            {
                $configValid = false;
                $errors[] = $tokenContext['message'];
            }
            $paymentOptionsContext = $this->getPaymentOptions();
            if ($paymentOptionsContext['state'] == false && $tokenContext['state'] == true)
            {
                $configValid = false;
                $errors[]    = $paymentOptionsContext['message'];
            }
            return ['state' => $configValid, 'message' => $errors];
        }
    }
    
    /**
     * getSecurityToken
     *
     * Create a security Token STS
     * 
     * @return array
     */
    private function getSecurityToken()
    {
        if (!empty($this->token) && $this->tokenEnv == $this->env) {
            if ((time() - $this->tokenUpdateDate) < 86400) {
                return array('state' => true, 'token' => $this->token);
            }
        }
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_TIMEOUT_MS, $this->timeoutMs);
        curl_setopt($ch, CURLOPT_HEADER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_URL, $this->contextPaymentUri.'v1/auth/token');
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'authorization: '.$this->authorization
        ));
        $response       = curl_exec($ch);
        $header_size    = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
        $body           = str_replace('"','',substr($response, $header_size));
        $httpcode       = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        if ($httpcode == 200)
        {
            $this->token            = $body;
            $this->tokenUpdateDate  = time();
            $this->tokenEnv         = $this->env;
            FloaPay::updateConfKey('FLOA_TK_TOKEN', $this->token);
            FloaPay::updateConfKey('FLOA_TK_TOKEN_UPDATE', $this->tokenUpdateDate);    
            FloaPay::updateConfKey('FLOA_TK_TOKEN_ENV', $this->env);
            return array('state' => true, 'token' => $this->token);
        } else {
            $message = "";
            switch($httpcode)
            {
                case 403:
                    $message = '[TOKEN] Your credentials are incorrect, access denied.';
                    break;
                case 500:
                    $message = '[TOKEN] Internal server error on FLOA environnement.';
                    break;
                case 400: 
                    $message = '[TOKEN] You have send a bad request on FLOA environnement.';
                    break;
                default:
                    $message = '[TOKEN] Unknown error on FLOA environment.';
                    break;
            }
            floaLogger::error($message);
            return array(
                'state'     => false, 
                'message'   => $message
            );
        }
    }
    
    /**
     * getEligibility
     *
     * Get the eligibility of a customer
     * 
     * @param  Cart $cart
     * @param  Address $shippingAddress
     * @param  Address $billingAddress
     * @param  array $eligibilityDatas
     * @param  integer $reportDelayInDays
     * @return array
     */
    public function getEligibility($cart, $shippingAddress, $billingAddress, $eligibilityDatas, $reportDelayInDays = 0)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_TIMEOUT_MS, $this->timeoutMs);
        curl_setopt($ch, CURLOPT_HEADER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_URL, $this->contextEligibilityUri.'api/v4/eligibilities/?merchantId='.$this->merchantId.'&merchantSiteIds='.$this->merchantSiteId);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'authToken: '.$this->token,
            'Content-Type: application/json',
            'accept: application/json'
        ));
        $orderRef = floaTools::generateOrderRef($this->method, $cart->id);
        $carrierMappingModel       = new floaCarrierMapping();
        $shippingMethod            = $carrierMappingModel->getFloaIdByPsId($cart->id_carrier);
        $eligibilityRequest = [
            "presaleFolder" => 
            [
                "rawAmount"         => floaTools::convertToInt($cart->getOrderTotal(true, Cart::BOTH)),
                "shoppingCarts"     => eligibilityHelper::shoppingCarts($cart, $shippingAddress, $orderRef, $shippingMethod),
                "merchantSite"      => eligibilityHelper::merchantSite($cart),
                "customer"          => eligibilityHelper::customer($billingAddress, $eligibilityDatas),
                "shippingAddress"   => eligibilityHelper::address($shippingAddress),
                "billingAddress"    => eligibilityHelper::address($billingAddress),
                "saleChannel"       => eligibilityHelper::saleChannel(),
                "isSecured"         => true,
                "reportDelayInDays" => $reportDelayInDays
            ]
        ];
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($eligibilityRequest));
        $response       = curl_exec($ch);
        $header_size    = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
        $body           = substr($response, $header_size);
        $httpcode       = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        if ($httpcode == 200)
        {
            $responseJson = json_decode($body);
            if (isset($responseJson->eligibilities[0]->hasAgreement) && $responseJson->eligibilities[0]->hasAgreement == true) {
                return array(
                    'state' => true,
                    'datas' => [
                        "token"         => $responseJson->eligibilities[0]->token,
                        "hasAgreement"  => $responseJson->eligibilities[0]->hasAgreement,
                        "totalAmount"   => $responseJson->eligibilities[0]->totalAmount,
                        "requestId"     => $responseJson->eligibilities[0]->requestId,
                        "orderRef"      => $orderRef,
                        "links"         => $responseJson->eligibilities[0]->links
                    ]
                );
            } else {
                if (isset($responseJson->eligibilities[0]->errors[0])) {
                    floaLogger::error("[ELIGIBILITY] ".$responseJson->eligibilities[0]->errors[0]);
                    $error = $responseJson->eligibilities[0]->errors[0];
                } else {
                    $error = '';
                }
                return array(
                    'state'     => false, 
                    'message'   => "[ELIGIBILITY] ".$error
                );
            }            
        } else {
            $message = "";
            switch($httpcode)
            {
                case 400: 
                    $message = '[ELIGIBILITY] You have send a bad request on FLOA environnement.';
                    break;
                case 401: 
                    $message = '[ELIGIBILITY] You are not authorized to make this request on FLOA environnement.';
                    break;
                case 403:
                    $message = '[ELIGIBILITY] Your credentials are incorrect, access denied.';
                    break;
                case 406:
                    $message = '[ELIGIBILITY] Your request is not acceptable.';
                    break;
                case 500:
                    $message = '[ELIGIBILITY] Internal server error on FLOA environnement.';
                    break;
                default:
                    $message = '[ELIGIBILITY] Unknown error on FLOA environment.';
                    break;
            }
            floaLogger::error($message);
            return array(
                'state'     => false, 
                'message'   => $message
            );
        }
    }
    
    public function getPaymentResultFromPaymentSchedules($orderRef, $merchantId = false, $merchantSiteId = false)
    {
        $schedules = $this->getPaymentSchedules($orderRef, $merchantId, $merchantSiteId);
        if (isset($schedules["state"]) && $schedules["state"] == true && isset($schedules["schedules"])) {
            $totalAmount = 0;
            foreach ($schedules["schedules"] as $schedule) {
                $totalAmount += $schedule->amount;
                if ($schedule->rank == 1) {
                    $state = $schedule->state;
                }
            }
            if (isset($state)) {
                switch ($state) {
                    case 'cancelled':
                    case 'init':
                        $paymentResultCode = 2;
                    break;
                    case 'payed':
                    case 'cashed':
                        $paymentResultCode = 0;
                    break;
                    default:
                        $paymentResultCode = 3;
                    break;
                }
            } else {
                $paymentResultCode = 3;
            }
            return [
                'state'     => true, 
                'datas'     => [
                    'amount'                => $totalAmount,
                    'paymentResultCode'     => $paymentResultCode
                ]
            ];
        } else {
            return [
                'state'     => false
            ];
        }
    }

    /**
     * getPaymentSchedules
     *
     * Get information form an existing Order
     * 
     * @param  string $orderRef
     * @param  integer $merchantId
     * @param  integer $merchantSiteId
     * @return array
     */
    public function getPaymentSchedules($orderRef, $merchantId = false, $merchantSiteId = false)
    {
        $merchantId     = $merchantId ? $merchantId : $this->merchantId;
        $merchantSiteId = $merchantSiteId ? $merchantSiteId : $this->merchantSiteId;
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_TIMEOUT_MS, $this->timeoutMs);
        curl_setopt($ch, CURLOPT_HEADER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_URL, $this->contextPaymentUri.'v1/payments/'.$orderRef.'/merchants/'.$merchantId.'/sites/'.$merchantSiteId);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'authToken: '.$this->token,
            'Content-Type: application/json',
            'accept: application/json'
        ));
        $response       = curl_exec($ch);
        $header_size    = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
        $body           = substr($response, $header_size);
        $httpcode       = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        if ($httpcode == 200)
        {
            $responseJson = json_decode($body);
            if (isset($responseJson->responseCode) && $responseJson->responseCode == "success")
            {
                return array('state' => true, 'schedules' => $responseJson->schedules);
            } else {
                floaLogger::error('[PAYMENT SCHEDULES] No more information.');
                return array(
                    'state' => false,
                    'message' => '[PAYMENT SCHEDULES] No more information.'
                );
            }
        } else {
            $message = "";
            switch($httpcode)
            {
                case 400: 
                    $message = '[PAYMENT SCHEDULES] You have send a bad request on FLOA environnement.';
                    break;
                case 401: 
                    $message = '[PAYMENT SCHEDULES] You are not authorized to make this request on FLOA environnement.';
                    break;
                case 403:
                    $message = '[PAYMENT SCHEDULES] Your credentials are incorrect, access denied.';
                    break;
                case 500:
                    $message = '[PAYMENT SCHEDULES] Internal server error on FLOA environnement.';
                    break;
                default:
                    $message = '[PAYMENT SCHEDULES] Unknown error on FLOA environment.';
                    break;
            }
            floaLogger::error($message);
            return array(
                'state'     => false, 
                'message'   => $message
            );
        }

    }
    
    /**
     * createPaymentSession
     *
     * Create a payment session 
     * 
     * @param  Cart $cart
     * @param  Address $shippingAddress
     * @param  Address $billingAddress
     * @param  array $scoringDatas
     * @param  integer $reportDelayInDays
     * @return array
     */
    public function createPaymentSession($country, $cart, $shippingAddress, $billingAddress, $scoringDatas, $reportDelayInDays = 0)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_TIMEOUT_MS, $this->timeoutMs);
        curl_setopt($ch, CURLOPT_HEADER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_URL, $this->contextPaymentUri.'v1/payment-sessions');
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'authToken: '.$this->token,
            'Content-Type: application/json',
            'accept: application/json'
        ));
        $bodyInfo = [
            "merchantId"     => $this->merchantId,
            "merchantSiteId" => $this->merchantSiteId,
            "customer"       => paymentSessionHelper::customer($billingAddress, $shippingAddress, $cart->id_customer, $country),
            "orderData"      => paymentSessionHelper::orderData($shippingAddress, $scoringDatas, $cart),
            "configuration"  => paymentSessionHelper::configuration($country, $cart, $this->defaultPaymentOptionRef, $reportDelayInDays)
        ];
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($bodyInfo));
        $response       = curl_exec($ch);
        $header_size    = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
        $body           = substr($response, $header_size);
        $httpcode       = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        if ($httpcode == 200)
        {
            $responseJson = json_decode($body);
            if (isset($responseJson->operationSucceeded) && $responseJson->operationSucceeded == true)
            {
                return array('state' => true, 'session' => $responseJson);
            } else {
                floaLogger::error('[PAYMENT SESSION] No more information.');
                return array(
                    'state' => false,
                    'message' => '[PAYMENT SESSION] No more information.'
                );
            }
        } else {
            $message = "";
            switch($httpcode)
            {
                case 400: 
                    $message = '[PAYMENT SESSION] You have send a bad request on FLOA environnement.';
                    break;
                case 401: 
                    $message = '[PAYMENT SESSION] You are not authorized to make this request on FLOA environnement.';
                    break;
                case 403:
                    $message = '[PAYMENT SESSION] Your credentials are incorrect, access denied.';
                    break;
                case 500:
                    $message = '[PAYMENT SESSION] Internal server error on FLOA environnement.';
                    break;
                default:
                    $message = '[PAYMENT SESSION] Unknown error on FLOA environment.';
                    break;
            }
            floaLogger::error($message);
            return array(
                'state'     => false, 
                'message'   => $message
            );
        }
    }
    
    /**
     * getPaymentResult
     *
     * Get result of an existing payment request
     * 
     * @param  string $payemntSessionID
     * @return array
     */
    public function getPaymentResult($payemntSessionID)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_TIMEOUT_MS, $this->timeoutMs);
        curl_setopt($ch, CURLOPT_HEADER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_URL, $this->contextPaymentUri.'v1/payment-sessions/'.$payemntSessionID.'/paymentResult/');
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'authToken: '.$this->token,
            'Content-Type: application/json',
            'accept: application/json'
        ));
        $response       = curl_exec($ch);
        $header_size    = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
        $body           = substr($response, $header_size);
        $httpcode       = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        if ($httpcode == 200)
        {
            $responseJson = json_decode($body, true);
            return array(
                'state'     => true, 
                'datas'     => $responseJson
            );
        } else {
            $message = "";
            switch($httpcode)
            {
                case 400: 
                    $message = '[PAYMENT RESULT] You have send a bad request on FLOA environnement.';
                    break;
                case 401: 
                    $message = '[PAYMENT RESULT] You are not authorized to make this request on FLOA environnement.';
                    break;
                case 403:
                    $message = '[PAYMENT RESULT] Your credentials are incorrect, access denied.';
                    break;
                case 500:
                    $message = '[PAYMENT RESULT] Internal server error on FLOA environnement.';
                    break;
                default:
                    $message = '[PAYMENT RESULT] Unknown error on FLOA environment.';
                    break;
            }
            floaLogger::error($message);
            return array(
                'state'     => false, 
                'message'   => $message
            );
        }
    }
    
    /**
     * getPaymentOptions
     * 
     * Get the payments options
     *
     * @return array
     */
    private function getPaymentOptions()
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_TIMEOUT_MS, $this->timeoutMs);
        curl_setopt($ch, CURLOPT_HEADER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_URL, $this->contextPaymentUri.'v1/payment-options/merchants/'.$this->merchantId.'/sites/'.$this->merchantSiteId);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'authToken: '.$this->token
        ));
        $response       = curl_exec($ch);
        $header_size    = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
        $body           = substr($response, $header_size);
        $httpcode       = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        if ($httpcode == 200)
        {
            $responseJson = json_decode($body);
            if (isset($responseJson->paymentOptions))
            {            
                foreach ($responseJson->paymentOptions as $paymentOption)
                {
                    if ($paymentOption->paymentOptionRef == $this->defaultPaymentOptionRef)
                    {
                        $this->paymentOption = $paymentOption;
                        $this->minAmount = $paymentOption->minAmount / 100;
                        $this->maxAmount = $paymentOption->maxAmount / 100;
                        return array('state' => true, 'paymentOption' => $this->paymentOption);
                    }
                }
            } else {
                floaLogger::error('[PAYMENT OPTIONS] No more information.');
                return array(
                    'state' => false,
                    'message' => '[PAYMENT OPTIONS] No more information.'
                );
            }
        } else {
            $message = "";
            switch($httpcode)
            {
                case 400: 
                    $message = '[PAYMENT OPTIONS] You have send a bad request on FLOA environnement.';
                    break;
                case 401: 
                    $message = '[PAYMENT OPTIONS] You are not authorized to make this request on FLOA environnement.';
                    break;
                case 403:
                    $message = '[PAYMENT OPTIONS] Your credentials are incorrect, access denied.';
                    break;
                case 500:
                    $message = '[PAYMENT OPTIONS] Internal server error on FLOA environnement.';
                    break;
                default:
                    $message = '[PAYMENT OPTIONS] Unknown error on FLOA environment.';
                    break;
            }
            floaLogger::error($message);
            return array(
                'state'     => false, 
                'message'   => $message
            );
        }
    }
    
    /**
     * getEstimatedSchedule
     * 
     * Get the estimated installment plan
     *
     * @param  float $amount
     * @return array
     */
    public function getEstimatedSchedule($amount, $reportDelayInDays = false)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_TIMEOUT_MS, $this->timeoutMs);
        curl_setopt($ch, CURLOPT_HEADER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_URL, $this->contextPaymentUri.'v1/estimated-schedule/merchants/'.$this->merchantId.'/sites/'.$this->merchantSiteId);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'authToken: '.$this->token,
            'Content-Type: application/json',
            'accept: application/json'
        ));
        $content = [
            'amount'                => floaTools::convertToInt($amount),
            'includePaymentFees'    => true,
            'orderDate'             => date('Y-m-d').'T'.date('H:i:s').'+00:00',
            'paymentOptionRef'      => $this->defaultPaymentOptionRef
        ];       
        if ($reportDelayInDays !== false) {
            $content['ReportDelayInDays'] = $reportDelayInDays;
        }
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($content));
        $response       = curl_exec($ch);
        $header_size    = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
        $body           = substr($response, $header_size);
        $httpcode       = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        if ($httpcode == 200)
        {
            $responseJson = json_decode($body);
            if (isset($responseJson->code) && $responseJson->code == 0)
            {
                return array(
                    'state'     => true, 
                    'amount'    => $responseJson->amount,
                    'schedules' => $responseJson->schedules
                );
            } else {
                floaLogger::error('[ESTIMATED SCHEDULE] '.(isset($responseJson->message) ? $responseJson->message : 'No more information.'));
                return array(
                    'state'     => false, 
                    'message'   => '[ESTIMATED SCHEDULE] '.(isset($responseJson->message) ? $responseJson->message : 'No more information.')
                );
            }
        } else {
            $message = "";
            switch($httpcode)
            {
                case 400: 
                    $message = '[ESTIMATED SCHEDULE] You have send a bad request on FLOA environnement.';
                    break;
                case 401: 
                    $message = '[ESTIMATED SCHEDULE] You are not authorized to make this request on FLOA environnement.';
                    break;
                case 403:
                    $message = '[ESTIMATED SCHEDULE] Your credentials are incorrect, access denied.';
                    break;
                case 500:
                    $message = '[ESTIMATED SCHEDULE] Internal server error on FLOA environnement.';
                    break;
                default:
                    $message = '[ESTIMATED SCHEDULE] Unknown error on FLOA environment.';
                    break;
            }
            floaLogger::error($message);
            return array(
                'state'     => false, 
                'message'   => $message
            );
        }
    }
    
    /**
     * updateOrder
     *
     * Update the total amount of an Order
     * 
     * @param  string $orderRef
     * @param  string $scoringToken
     * @param  integer $oldAmount
     * @param  integer $newAmount
     * @param  integer $merchantId
     * @param  integer $merchantSiteId
     * @return array
     */
    public function updateOrder($orderRef, $scoringToken, $oldAmount, $newAmount, $merchantId, $merchantSiteId)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_TIMEOUT_MS, $this->timeoutMs);
        curl_setopt($ch, CURLOPT_HEADER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_URL, $this->contextServicesUri.'UpdateOrder');
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'Content-Type: application/json',
            'accept: application/json'
        ));
        $content = [
            "headerMessage" => [
                "Context" => [
                    "MerchantId"        => $merchantId,
                    "MerchantSiteId"    => $merchantSiteId
                ],
                "SecurityContext" => [
                    "DomainRightsList"  => null,
                    "IssuerID"          => null,
                    "SessionID"         => null,
                    "SubjectLocality"   => null,          
                    "TokenId"           => $this->token,
                    "UserName"          => null
                ],
                "Localization" => [
                    "Language"          => "FR",
                    "Currency"          => "EUR",
                    "Country"           => "FR",
                    "DecimalPosition"   => 2
                ],
                "Version"   => "1.0"
            ],
            "updateOrderRequestMessage" => [
                "OrderRef"      => $orderRef,
                "ScoringToken"  => $scoringToken,
                "OldAmount"     => $oldAmount,
                "NewAmount"     => $newAmount
            ]
        ];
    
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($content));
        $response       = curl_exec($ch);
        $header_size    = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
        $body           = substr($response, $header_size);
        $httpcode       = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        if ($httpcode == 200)
        {
            $responseJson = json_decode($body);
            if (isset($responseJson->UpdateOrderResult)
                && isset($responseJson->UpdateOrderResult->ResponseCode)
                && ($responseJson->UpdateOrderResult->ResponseCode == 0 || $responseJson->UpdateOrderResult->ResponseCode == 4)) {
                    return array(
                        'state'     => true, 
                        'schedules' => $responseJson->UpdateOrderResult->Schedule,
                        'message'   => $responseJson->UpdateOrderResult->ResponseMessage
                    );
            } else {
                return array(
                    'state'     => false, 
                    'message'   => '[UPDATE ORDER] '.isset($responseJson->UpdateOrderResult->ResponseMessage) ? $responseJson->UpdateOrderResult->ResponseMessage : 'No more information'
                );
            }
        } else {
            $message = "";
            switch($httpcode)
            {
                case 400: 
                    $message = '[UPDATE ORDER] You have send a bad request on FLOA environnement.';
                    break;
                case 401: 
                    $message = '[UPDATE ORDER] You are not authorized to make this request on FLOA environnement.';
                    break;
                case 403:
                    $message = '[UPDATE ORDER] Your credentials are incorrect, access denied.';
                    break;
                case 500:
                    $message = '[UPDATE ORDER] Internal server error on FLOA environnement.';
                    break;
                default:
                    $message = '[UPDATE ORDER] Unknown error on FLOA environment.';
                    break;
            }
            floaLogger::error($message);
            return array(
                'state'     => false, 
                'message'   => $message
            );
        }
    }
    
    /**
     * payOrderRank
     *
     * Capture a valid Order
     *
     * @param  string $orderRef
     * @param  integer $merchantId
     * @param  integer $merchantSiteId
     * @return array
     */
    public function payOrderRank($orderRef, $merchantId, $merchantSiteId)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_TIMEOUT_MS, $this->timeoutMs);
        curl_setopt($ch, CURLOPT_HEADER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_URL, $this->contextPaymentUri.'v1/payments/'.$orderRef.'/operations/capture');
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'authToken: '.$this->token,
            'Content-Type: application/json',
            'accept: application/json'
        ));
        $content = [
            "merchantId"        => $merchantId,
            "merchantSiteId"    => $merchantSiteId,
            "attempt"           => 1,
            "rank"              => 1
        ];
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "PUT"); 
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($content));
        $response       = curl_exec($ch);
        $header_size    = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
        $body           = substr($response, $header_size);
        $httpcode       = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        if ($httpcode == 200)
        {
            $responseJson = json_decode($body);
            if (isset($responseJson->responseCode)
                && $responseJson->responseCode == 'success'
                && $responseJson->actionType == 'capture') {
                    return array(
                        'state'     => true, 
                        'orderRef' => $responseJson->orderRef
                    );
            } else {
                return array(
                    'state'     => false, 
                    'message'   => '[CAPTURE ORDER] '.isset($responseJson->responseMessage) ? $responseJson->responseMessage : 'No more information'
                );
            }
        } else {
            $message = "";
            switch($httpcode)
            {
                case 400: 
                    $message = '[CAPTURE ORDER] You have send a bad request on FLOA environnement.';
                    break;
                case 401: 
                    $message = '[CAPTURE ORDER] You are not authorized to make this request on FLOA environnement.';
                    break;
                case 403:
                    $message = '[CAPTURE ORDER] Your credentials are incorrect, access denied.';
                    break;
                case 500:
                    $message = '[CAPTURE ORDER] Internal server error on FLOA environnement.';
                    break;
                default:
                    $message = '[CAPTURE ORDER] Unknown error on FLOA environment.';
                    break;
            }
            floaLogger::error($message);
            return array(
                'state'     => false, 
                'message'   => $message
            );
        }
    }

}
