<?php
/**
 * 2021 Floa BANK
 *
 * THE MIT LICENSE
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * @author    FLOA Bank
 * @copyright 2021 FLOA Bank
 * @license   https://opensource.org/licenses/MIT The MIT License
 */

if(!defined('STDIN'))  define('STDIN',  fopen('php://stdin',  'rb'));
if(!defined('STDOUT')) define('STDOUT', fopen('php://stdout', 'wb'));
if(!defined('STDERR')) define('STDERR', fopen('php://stderr', 'wb'));

class floaLogger {

    /**
     * @var array
     */
    protected static $log = [];

    /**
     * @var bool
     */
    public static $print_log = true;

    /**
     * @var bool
     */
    public static $write_log = true;

    /**
     * @var string
     */
    public static $log_file_extension = "log";

    /**
     * @var bool
     */
    public static $log_file_append = true;

    /**
     * @var string
     */
    private static $log_file_path = '';

    /**
     * @var array
     */
    private static $output_streams = [];

    /**
     * @var bool
     */
    private static $logger_ready = false;

    /**
     * @var array
     */
    private static $time_tracking = [];
    
    /**
     * info
     *
     * @param  string $message
     * @param  string $name
     * @return void
     */
    public static function info($message, $name = '') {
        return self::add($message, $name, 'info');
    }
    
    /**
     * debug
     *
     * @param  string $message
     * @param  string $name
     * @return void
     */
    public static function debug($message, $name = '') {
        return self::add($message, $name, 'debug');
    }
    
    /**
     * warning
     *
     * @param  string $message
     * @param  string $name
     * @return void
     */
    public static function warning($message, $name = '') {
        return self::add($message, $name, 'warning');
    }
    
    /**
     * error
     *
     * @param  string $message
     * @param  string $name
     * @return void
     */
    public static function error($message, $name = '') {
        return self::add($message, $name, 'error');
    }
    
    /**
     * time
     *
     * @param  string $name
     * @return void
     */
    public static function time(string $name) {
        if (!isset(self::$time_tracking[$name])) {
            self::$time_tracking[$name] = microtime(true);
            return self::$time_tracking[$name];
        }
        else {
            return false;
        }
    }
    
    /**
     * timeEnd
     *
     * @param  string $name
     * @return void
     */
    public static function timeEnd(string $name) {
        if (isset(self::$time_tracking[$name])) {
            $start = self::$time_tracking[$name];
            $end = microtime(true);
            $elapsed_time = number_format(($end - $start), 2);
            unset(self::$time_tracking[$name]);
            self::add("$elapsed_time seconds", "'$name' took", "timing");
            return $elapsed_time;
        }
        else {
            return false;
        }
    }
    
    /**
     * add
     *
     * @param  string $message
     * @param  string $name
     * @param  string $level
     * @return void
     */
    private static function add($message, $name = '', $level = 'debug') {
        $log_entry = [
            'timestamp' => time(),
            'name' => $name,
            'message' => $message,
            'level' => $level,
       ];
        self::$log[] = $log_entry;
        if (!self::$logger_ready) {
            self::init();
        }
        if (self::$logger_ready && count(self::$output_streams) > 0) {
            $output_line = self::format_log_entry($log_entry).PHP_EOL;
            foreach (self::$output_streams as $key => $stream) {
                fputs($stream, $output_line);
            }
        }
        return $log_entry;
    }
    
    /**
     * format_log_entry
     *
     * @param  array $log_entry
     * @return string
     */
    public static function format_log_entry(array $log_entry) {
        $log_line = "";
        if (!empty($log_entry)) {
            $log_entry = array_map(function($v) { return print_r($v, true); }, $log_entry);
                    $log_line .= date('c', $log_entry['timestamp'])." ";
            $log_line .= "[".strtoupper($log_entry['level'])."] : ";
            if (!empty($log_entry['name'])) {
                $log_line .= $log_entry['name']." => ";
            }
            $log_line .= $log_entry['message'];
        
        }
        return $log_line;
    }
        
    /**
     * init
     *
     * @return void
     */
    public static function init() {
        if (!self::$logger_ready) {
            $log_dir = version_compare(_PS_VERSION_, '1.7', '>=') ? _PS_ROOT_DIR_.'/var/logs/' : _PS_ROOT_DIR_.'/log/';
            $log_file_name = date('Ymd')."_floapay";
            if (true === self::$print_log) {
                self::$output_streams['stdout'] = STDOUT;
            }
            if (file_exists($log_dir)) {
                self::$log_file_path = implode(DIRECTORY_SEPARATOR, [$log_dir, $log_file_name]);
                if (!empty(self::$log_file_extension)) {
                    self::$log_file_path .= ".".self::$log_file_extension;
                }
            }
            if (true === self::$write_log) {
                if (file_exists($log_dir)) {
                    $mode = self::$log_file_append ? "a" : "w";
                    self::$output_streams[self::$log_file_path] = fopen(self::$log_file_path, $mode);
                }
            }
        }
        self::$logger_ready = true;
    }
    
    /**
     * dump_to_file
     *
     * @param  mixed $file_path
     * @return void
     */
    public static function dump_to_file($file_path='') {

        if (!$file_path) {
            $file_path = self::$log_file_path;
        }
        if (file_exists(dirname($file_path))) {
            $mode = self::$log_file_append ? "a" : "w";
            $output_file = fopen($file_path, $mode);
            foreach (self::$log as $log_entry) {
                $log_line = self::format_log_entry($log_entry);
                fwrite($output_file, $log_line.PHP_EOL);
            }
            fclose($output_file);
        }
    }
        
    /**
     * dump_to_string
     *
     * @return string
     */
    public static function dump_to_string() {
        $output = '';
        foreach (self::$log as $log_entry) {
            $log_line = self::format_log_entry($log_entry);
            $output .= $log_line.PHP_EOL;
        }
        return $output;
    }
}
